package com.sun.jndi.toolkit.ctx;

import javax.naming.NameParser;
import javax.naming.Context;
import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.OperationNotSupportedException;
import javax.naming.NotContextException;
import javax.naming.CannotProceedException;
import javax.naming.spi.DirectoryManager;
import javax.naming.CompositeName;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.NamingEnumeration;
import javax.naming.directory.ModificationItem;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;
import javax.naming.Name;
import javax.naming.directory.DirContext;

public abstract class PartialCompositeDirContext extends AtomicContext implements DirContext
{
    protected PartialCompositeDirContext() {
        this._contextType = 1;
    }
    
    protected abstract Attributes p_getAttributes(Name p0, String[] p1, Continuation p2) throws NamingException;
    
    protected abstract void p_modifyAttributes(Name p0, int p1, Attributes p2, Continuation p3) throws NamingException;
    
    protected abstract void p_modifyAttributes(Name p0, ModificationItem[] p1, Continuation p2) throws NamingException;
    
    protected abstract void p_bind(Name p0, Object p1, Attributes p2, Continuation p3) throws NamingException;
    
    protected abstract void p_rebind(Name p0, Object p1, Attributes p2, Continuation p3) throws NamingException;
    
    protected abstract DirContext p_createSubcontext(Name p0, Attributes p1, Continuation p2) throws NamingException;
    
    protected abstract NamingEnumeration<SearchResult> p_search(Name p0, Attributes p1, String[] p2, Continuation p3) throws NamingException;
    
    protected abstract NamingEnumeration<SearchResult> p_search(Name p0, String p1, SearchControls p2, Continuation p3) throws NamingException;
    
    protected abstract NamingEnumeration<SearchResult> p_search(Name p0, String p1, Object[] p2, SearchControls p3, Continuation p4) throws NamingException;
    
    protected abstract DirContext p_getSchema(Name p0, Continuation p1) throws NamingException;
    
    protected abstract DirContext p_getSchemaClassDefinition(Name p0, Continuation p1) throws NamingException;
    
    @Override
    public Attributes getAttributes(String s) throws NamingException {
        return this.getAttributes(s, (String[])null);
    }
    
    @Override
    public Attributes getAttributes(Name name) throws NamingException {
        return this.getAttributes(name, (String[])null);
    }
    
    @Override
    public Attributes getAttributes(String n, String[] array) throws NamingException {
        return this.getAttributes(new CompositeName(n), array);
    }
    
    @Override
    public Attributes getAttributes(Name name, String[] array) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        Attributes attributes;
        try {
            attributes = this.p_getAttributes(name, array, continuation);
            while (continuation.isContinue()) {
                attributes = getPCDirContext(continuation).p_getAttributes(continuation.getRemainingName(), array, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            attributes = DirectoryManager.getContinuationDirContext(cpe).getAttributes(cpe.getRemainingName(), array);
        }
        return attributes;
    }
    
    @Override
    public void modifyAttributes(String n, int n2, Attributes attributes) throws NamingException {
        this.modifyAttributes(new CompositeName(n), n2, attributes);
    }
    
    @Override
    public void modifyAttributes(Name name, int n, Attributes attributes) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_modifyAttributes(name, n, attributes, continuation);
            while (continuation.isContinue()) {
                getPCDirContext(continuation).p_modifyAttributes(continuation.getRemainingName(), n, attributes, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            DirectoryManager.getContinuationDirContext(cpe).modifyAttributes(cpe.getRemainingName(), n, attributes);
        }
    }
    
    @Override
    public void modifyAttributes(String n, ModificationItem[] array) throws NamingException {
        this.modifyAttributes(new CompositeName(n), array);
    }
    
    @Override
    public void modifyAttributes(Name name, ModificationItem[] array) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_modifyAttributes(name, array, continuation);
            while (continuation.isContinue()) {
                getPCDirContext(continuation).p_modifyAttributes(continuation.getRemainingName(), array, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            DirectoryManager.getContinuationDirContext(cpe).modifyAttributes(cpe.getRemainingName(), array);
        }
    }
    
    @Override
    public void bind(String n, Object o, Attributes attributes) throws NamingException {
        this.bind(new CompositeName(n), o, attributes);
    }
    
    @Override
    public void bind(Name name, Object o, Attributes attributes) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_bind(name, o, attributes, continuation);
            while (continuation.isContinue()) {
                getPCDirContext(continuation).p_bind(continuation.getRemainingName(), o, attributes, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            DirectoryManager.getContinuationDirContext(cpe).bind(cpe.getRemainingName(), o, attributes);
        }
    }
    
    @Override
    public void rebind(String n, Object o, Attributes attributes) throws NamingException {
        this.rebind(new CompositeName(n), o, attributes);
    }
    
    @Override
    public void rebind(Name name, Object o, Attributes attributes) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_rebind(name, o, attributes, continuation);
            while (continuation.isContinue()) {
                getPCDirContext(continuation).p_rebind(continuation.getRemainingName(), o, attributes, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            DirectoryManager.getContinuationDirContext(cpe).rebind(cpe.getRemainingName(), o, attributes);
        }
    }
    
    @Override
    public DirContext createSubcontext(String n, Attributes attributes) throws NamingException {
        return this.createSubcontext(new CompositeName(n), attributes);
    }
    
    @Override
    public DirContext createSubcontext(Name name, Attributes attributes) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        DirContext dirContext;
        try {
            dirContext = this.p_createSubcontext(name, attributes, continuation);
            while (continuation.isContinue()) {
                dirContext = getPCDirContext(continuation).p_createSubcontext(continuation.getRemainingName(), attributes, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            dirContext = DirectoryManager.getContinuationDirContext(cpe).createSubcontext(cpe.getRemainingName(), attributes);
        }
        return dirContext;
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String s, Attributes attributes) throws NamingException {
        return this.search(s, attributes, (String[])null);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes attributes) throws NamingException {
        return this.search(name, attributes, (String[])null);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String n, Attributes attributes, String[] array) throws NamingException {
        return this.search(new CompositeName(n), attributes, array);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, Attributes attributes, String[] array) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        NamingEnumeration<SearchResult> namingEnumeration;
        try {
            namingEnumeration = this.p_search(name, attributes, array, continuation);
            while (continuation.isContinue()) {
                namingEnumeration = getPCDirContext(continuation).p_search(continuation.getRemainingName(), attributes, array, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            namingEnumeration = DirectoryManager.getContinuationDirContext(cpe).search(cpe.getRemainingName(), attributes, array);
        }
        return namingEnumeration;
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String n, String s, SearchControls searchControls) throws NamingException {
        return this.search(new CompositeName(n), s, searchControls);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String s, SearchControls searchControls) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        NamingEnumeration<SearchResult> namingEnumeration;
        try {
            namingEnumeration = this.p_search(name, s, searchControls, continuation);
            while (continuation.isContinue()) {
                namingEnumeration = getPCDirContext(continuation).p_search(continuation.getRemainingName(), s, searchControls, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            namingEnumeration = DirectoryManager.getContinuationDirContext(cpe).search(cpe.getRemainingName(), s, searchControls);
        }
        return namingEnumeration;
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(String n, String s, Object[] array, SearchControls searchControls) throws NamingException {
        return this.search(new CompositeName(n), s, array, searchControls);
    }
    
    @Override
    public NamingEnumeration<SearchResult> search(Name name, String s, Object[] array, SearchControls searchControls) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        NamingEnumeration<SearchResult> namingEnumeration;
        try {
            namingEnumeration = this.p_search(name, s, array, searchControls, continuation);
            while (continuation.isContinue()) {
                namingEnumeration = getPCDirContext(continuation).p_search(continuation.getRemainingName(), s, array, searchControls, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            namingEnumeration = DirectoryManager.getContinuationDirContext(cpe).search(cpe.getRemainingName(), s, array, searchControls);
        }
        return namingEnumeration;
    }
    
    @Override
    public DirContext getSchema(String n) throws NamingException {
        return this.getSchema(new CompositeName(n));
    }
    
    @Override
    public DirContext getSchema(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        DirContext dirContext;
        try {
            dirContext = this.p_getSchema(name, continuation);
            while (continuation.isContinue()) {
                dirContext = getPCDirContext(continuation).p_getSchema(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            dirContext = DirectoryManager.getContinuationDirContext(cpe).getSchema(cpe.getRemainingName());
        }
        return dirContext;
    }
    
    @Override
    public DirContext getSchemaClassDefinition(String n) throws NamingException {
        return this.getSchemaClassDefinition(new CompositeName(n));
    }
    
    @Override
    public DirContext getSchemaClassDefinition(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        DirContext dirContext;
        try {
            dirContext = this.p_getSchemaClassDefinition(name, continuation);
            while (continuation.isContinue()) {
                dirContext = getPCDirContext(continuation).p_getSchemaClassDefinition(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            dirContext = DirectoryManager.getContinuationDirContext(cpe).getSchemaClassDefinition(cpe.getRemainingName());
        }
        return dirContext;
    }
    
    protected static PartialCompositeDirContext getPCDirContext(Continuation continuation) throws NamingException {
        PartialCompositeContext pcContext = PartialCompositeContext.getPCContext(continuation);
        if (!(pcContext instanceof PartialCompositeDirContext)) {
            throw continuation.fillInException(new NotContextException("Resolved object is not a DirContext."));
        }
        return (PartialCompositeDirContext)pcContext;
    }
    
    @Override
    protected StringHeadTail c_parseComponent(String s, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected Object a_lookup(String s, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected Object a_lookupLink(String s, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected NamingEnumeration<NameClassPair> a_list(Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected NamingEnumeration<Binding> a_listBindings(Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected void a_bind(String s, Object o, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected void a_rebind(String s, Object o, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected void a_unbind(String s, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected void a_destroySubcontext(String s, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected Context a_createSubcontext(String s, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected void a_rename(String s, Name name, Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
    
    @Override
    protected NameParser a_getNameParser(Continuation continuation) throws NamingException {
        throw continuation.fillInException(new OperationNotSupportedException());
    }
}
